#!/usr/bin/env python

# Converts a z80 source to source that can be imported to Orgams (an Amstrad CPC assembler):
# - Suppresses the ":" at the end of the labels.
# - Replaces DB by BYTE, DW by WORD, DS to FILL.
# - Replaces ENDIF by END.
# - Removes the ASSERT lines.
# - ORG and ENT $ can be added at the beginning.
# - Comments can be removed.
# Requires Python >= 3.
from __future__ import print_function
import sys

import argparse

def main(arguments):
        # Sets the command line parser.
        parser = argparse.ArgumentParser()
        parser.add_argument("-rc", "--removeComments", help="If present, removes all the comments.", action='store_const', const=True)
        parser.add_argument("-oe", "--orgent", help="If present, adds an ORG <ORGENT>/ENT $. The given value is copied verbatim after the ORG. You may use \"\" to use characters such as \"&\".", type=str, required=False)
        parser.add_argument("-i", "--input", help="The input file", type=str, required=True)
        parser.add_argument("-o", "--output", help="The output file", type=str, required=True)
        args = parser.parse_args(arguments)
        
        outputLines = []
        
        # Encodes the ORG/ENT if wanted.
        if (args.orgent):
                outputLines.append("".join(["        ORG ", args.orgent, "\n"]))
                outputLines.append("        ENT $\n\n")
        
        # Opens the file.
        with open(args.input) as f:
                inputLines = f.readlines()
                for line in inputLines:
                        mustEncodeLine = True
                        
                        # Replaces all the tabs by a whitespace (security).
                        line = line.replace("\t", ' ')
			
                        # Removes all the several whitespaces between all the words of a line.
                        beginWithSpace = line.startswith(' ')
                        line = ' '.join(line.split())
                        # However it also removes the possible first whitespace. Put it back.
                        if beginWithSpace:
                                line = ' ' + line

                        # Gets the first and second words of the line.
                        splits = line.split(' ')
                        firstWord = splits[0].strip()                   # Strip is necessary, else the ":" might now be found.
                        
                        if len(splits) > 1:
                                secondWord = splits[1].strip()
                        else:
                                secondWord = ""

                        # Assertion? Removes the line if yes.
                        mustEncodeLine = not line.strip().startswith("assert")
                                
                        if mustEncodeLine:
                                # Label? Remove the trailing ":".
                                if firstWord.endswith(":"):
                                        line = line.replace(firstWord, firstWord[:-1], 1)
                                
                                # DB? DW? DS? ENDIF? May be second word too. Not very elegant, who cares...
                                if firstWord.lower() == "db":
                                        line = line.replace(firstWord, "BYTE", 1)
                                elif firstWord.lower() == "dw":
                                        line = line.replace(firstWord, "WORD", 1)
                                elif firstWord.lower() == "ds":
                                        line = line.replace(firstWord, "FILL", 1)
                                # Second word.
                                elif secondWord.lower() == "db":
                                        line = line.replace(secondWord, "BYTE", 1)
                                elif secondWord.lower() == "dw":
                                        line = line.replace(secondWord, "WORD", 1)
                                elif secondWord.lower() == "ds":
                                        line = line.replace(secondWord, "FILL", 1)
                                elif secondWord.lower() == "endif":
                                        line = line.replace(secondWord, "END", 1)

                                # Removes the comments?
                                if args.removeComments:
                                        index = line.find(';')
                                        if index >= 0:
                                                line = line[0:index]
                                                # If, after removing the comment, it is an empty line, no need to encode it.
                                                mustEncodeLine = (line.strip() != '')
                                
                        # Encodes the line, if we must.
                        if mustEncodeLine:
                                outputLines.append(line)
                                outputLines.append("\n")
        
        # Writes the lines into the output file.
        with open(args.output, "w") as fo:
                for line in outputLines:
                        fo.write(line)
                
	
if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))